from queue import Queue

from era_5g_interface.task_handler_gstreamer import TaskHandlerGstreamer
from era_5g_interface.task_handler_internal_q import TaskHandlerInternalQ


class TaskHandlerGstreamerInternalQ(TaskHandlerGstreamer, TaskHandlerInternalQ):
    """Task handler which combines the Gstreamer functionality of data
    retrieval with usage of python internal queues for passing the data to the
    worker object."""

    def __init__(self, sid: str, port: int, image_queue: Queue, **kw) -> None:
        """
        Constructor
        Args:
            sid (str): The session id obtained from NetApp client. It is used to
                match the results with the data sender.
            port (int): The port where the Gstreamer pipeline should listen to.
            image_queue (Queue): The queue where the image and metadata should
                be passed to.
        """

        super().__init__(sid=sid, port=port, image_queue=image_queue, **kw)



import logging
import time
from abc import ABC

import cv2

from era_5g_interface.task_handler import TaskHandler, TaskHandlerInitializationFailed


class TaskHandlerGstreamer(TaskHandler, ABC):
    """Abstract class.

    Task handler which takes care of reading the data from Gstreamer
    pipeline with defined parameters. It needs to be inherited to
    implement the store_image method.
    """

    def __init__(self, sid: str, port: int, **kw) -> None:
        """
        Constructor
        Args:
            sid (str): The session id obtained from NetApp client. It is used to
                match the results with the data sender.
            port (int): The port where the Gstreamer pipeline should listen to.
        """

        super().__init__(sid=sid, **kw)
        self.port = port

    def run(self) -> None:
        """The infinite loop which reads the Gstreamer pipeline and pass the
        images to the store_image method, which has to be implemented in the
        child class.

        Raises:
            TaskHandlerInitializationFailed: Raised when the construction of
                Gstreamer pipeline failed
        """

        # pipeline which decodes a h264 stream
        pipeline = (
            f"udpsrc port={self.port} "
            'caps="application/x-rtp,media=(string)video,encoding-name=(string)H264,'
            'payload=(int)96" ! '
            "rtph264depay ! avdec_h264 ! videoconvert ! appsink"
        )

        try:
            logging.info(f"Creating Gstreamer capture on port {self.port}")
            # standard OpenCV VideoCapture connected to the Gstreamer pipeline
            cap = cv2.VideoCapture(pipeline)
            if not cap.isOpened():
                raise TaskHandlerInitializationFailed("VideoCapture was not opened")
            logging.info("Gstreamer capture created")
        except Exception as e:
            logging.error(f"Gstreamer capture failed. {str(e)}")
            # TODO: raise exception?
            exit(1)

        while not self.stop_event.is_set():
            ret, frame = cap.read()
            if ret:
                recv_timestamp = time.time_ns()
                # use frame number instead of unknown timestamp
                logging.debug(f"CAP_PROP_POS_MSEC: {int(cap.get(cv2.CAP_PROP_POS_MSEC ))}")
                logging.debug(f"CAP_PROP_POS_FRAMES: {int(cap.get(cv2.CAP_PROP_POS_FRAMES))} {recv_timestamp}")
                timestamp = str(int(cap.get(cv2.CAP_PROP_POS_FRAMES)))
                self.store_image(
                    {
                        "sid": self.sid,
                        "websocket_id": self.websocket_id,
                        "timestamp": timestamp,
                        "recv_timestamp": recv_timestamp,
                    },
                    frame,
                )
        cap.release()



from queue import Empty, Full, Queue

import numpy as np

from era_5g_interface.dataclasses.control_command import ControlCommand
from era_5g_interface.task_handler import TaskHandler


class TaskHandlerInternalQ(TaskHandler):
    """Task handler which takes care of passing the data to the python internal
    queue for future processing.

    It could either be inherited to implement the _run method and read
    the data from any source or used directly and call the store_image
    method externally.
    """

    def __init__(self, sid: str, image_queue: Queue, **kw) -> None:
        """
        Constructor
        Args:
            sid (str): The session id obtained from NetApp client. It is used to
                match the results with the data sender.
            image_queue (Queue): The queue where the image and metadata should
                be passed to.
        """

        super().__init__(sid=sid, **kw)
        self._q = image_queue
        self.index = 0

    def store_image(self, metadata: dict, image: np.ndarray) -> None:
        """Method which will store the image to the queue for processing.

        Args:
            metadata (dict): Arbitrary dictionary with metadata related to the image.
                The format is NetApp-specific.
            image (_type_): The image to be processed.
        """

        try:
            self._q.put((metadata, image), block=False)
        except Full:
            pass
            # TODO: raise an exception

    def store_control_data(self, data: ControlCommand) -> None:
        """Pass control commands to the worker using internal queue.

        Args:
            data (ControlCommand): ControlCommand with control data.
        """

        try:
            self._q.put((data), block=False)
        except Full:
            pass
            # TODO: raise an exception

    def clear_storage(self) -> None:
        """Clear all items from internal queue."""

        while not self._q.empty():
            try:
                self._q.get(block=False)
            except Empty:
                break
            self._q.task_done()

    def run(self) -> None:
        pass
